% ---------------------------------------------------------------------------
% -- This MATLAB file was developed by Daniel Llamocca.  It may be
% -- freely copied and/or distributed at no cost.  Any persons using this
% -- file for any purpose do so at their own risk, and are responsible for
% -- the results of such use.  Daniel Llamocca does not guarantee that
% -- this file is complete, correct, or fit for any particular purpose.
% -- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
% -- accompany any copy of this file.
% --------------------------------------------------------------------------

clear all; close all; clc;

% The following is an example for the 1D FIR VHDL implementation.
% The user can create a different filter with different parameters.

  N = 32; % Number of coefficients
  symmetry = 'a'; % Symmetry of the coefficients: symmetry = 'n', 'y'
  
  % The LUT values depend on N, NH, and L
    NH = 16; % Number of bits per coefficients
    B = 8; % Number of bits per input sample
    NO = 8; NOQ = NO-1;
    NX = 480; % Length of input stream
    L = 4; % Any L >1 is allowed. Be careful that only L = 4, 5, 6,7,8
           % are allowed in FIR_DA core when USE_PRIM = "YES"
           % Also:
           % If symmetry = 'y','a' --> L has to be multiple of ceil(N/2)
           % if symmetry = 'n' --> L has to be multiple of N
  op = 0; % Output rounding scheme, op = 0 (LSB truncation, MSB saturation)
  nstreams = 3; % number of streams to test. Useful to test the filters
  
% Generation of the input  
   x = gen_input(NX, nstreams, B, 'yes', 'stc'); % 'x': input quantized with [B B-1]            
   
% Generation of the Filter coefficients:
% **************************************
  gaussi = fspecial ('gaussian', N, 1.5); % This is a 2D filter
  [U,S,V] = svd (gaussi);
  v = U(:,1)*sqrt(S(1,1));
  h = V(:,1)'*sqrt(S(1,1)); % We will only use this 1D filter.

  % If the parameter 'symmetry' is 'a', we are forcing the
  % creation of an antisymmetric filter for testing purposes
  if symmetry == 'a' % antisymmetric
      h(1:floor(N/2)) = -h(1:floor(N/2)); % antisymmetric filter
  end

  % Plotting the filter response
    freqz (h, 1,256); title ('Magnitude response');
    figure; impz(h,1); title ('Impulse response');
    disp ('...'); disp ('Filter implemented!');
  
% Creation of the LUT files of the coefficients (used by the VHDL code)  
% *********************************************
   hh{1} = h; % we only use the horizontal 1D filter of the Gaussian.
   header = strcat('LUT_valuesN_',num2str(N),'_NH',num2str(NH));
   gen_txt_LUT(NH,L,N,hh, symmetry,header);
   disp ('Success!');
   
% Parameters for the VHDL code (main file: FIR_DA.vhd) :
% B = 8; op = 0; NO = 16; NQ = 15; USE_PRIM = "YES";
% symmetry = 'a'; file_LUT = 'LUT_valuesN_32_NH16.txt';
